/* ---------------------------------------------------------------------------
Programme : OPERTETE.C
Langage   : C sur UNIX
Date      : Janvier 1996 (Revision)

Auteur    : Francois Meunier

   Ce module comprend les operations relatives a la tete d'une image.

###########################################################################
#                                                                         #
#                          MODIFICATIONS                                  #
#                         ===============                                 #
#                                                                         #
#  Comme les images SunRaster ( *.ras, *.rast, *.RAS ou *.RAST) utilisent #
#  le format de donnees Big Endian, il est impossible de les utiliser     #
#  directement sur une plate-forme Windows ou Linux.                      #
#  Le format des donnees doit etre convertit au format Little Endian.     #
#  La difference reside dans l'ordre de lecture des octets.               #
#                                                                         #
#    Par : Gontran Fournier                                               #
#    Date: janvier 2000                                                   #
#                                                                         #
###########################################################################

--------------------------------------------------------------------------- */

#include "image.h"
#include <limits.h>

/* ---------------------------------------------------------------------------
Fonction LIRETETE : Cette fonction lit la tete (header) d'un fichier image
                    source (selon le descripteur 'fde').  Le parametre 'info'
permet l'affichage ou non de l'information contenue dans la tete du fichier.
Il y a aussi verification du 'magic number', du type de fichier 'raster' et
du type de la table des couleurs.

Valeurs possible du parametre 'info' :

        0 : n`affiche pas les informations de la tete (header)
        1 : affiche les informations de la tete (header)

appel : lireTete (image, 0 ou 1)
--------------------------------------------------------------------------- */
   void lireTete (Image *image, int info)
   {
   
      if ((image->tete = (TeteRaster *) malloc (sizeof (TeteRaster))) ==
         (TeteRaster *) NULL)
      {
         printf ("\nInfo : Memoire non disponible pour traitement (lireTete)...");
         exit(1);
      
      } /* if then */
   
      if (fread (image->tete, sizeof(TeteRaster), 1, image->fichiers.fde) != 1)
      {
         printf ("\nInfo : Impossible de lire la tete de '%s'.\n",
                *image->fichiers.fichierS);
         exit (-1);
      
      } /* if then */
   
   
   /*===========================================================================
    *     De Big Endian a Little Endian pour rouler sous WINDOWS
    *==========================================================================*/
      image->tete->rasMagic		 = switchByteOrder(image->tete->rasMagic      );
      image->tete->rasLargeur		 = switchByteOrder(image->tete->rasLargeur    );
      image->tete->rasHauteur		 = switchByteOrder(image->tete->rasHauteur    );
      image->tete->rasProfondeur	 = switchByteOrder(image->tete->rasProfondeur );
      image->tete->rasLongueur	 = switchByteOrder(image->tete->rasLongueur   );
      image->tete->rasType			 = switchByteOrder(image->tete->rasType       );
      image->tete->rasMapType		 = switchByteOrder(image->tete->rasMapType    );
      image->tete->rasMapLongueur = switchByteOrder(image->tete->rasMapLongueur);
   
   
      if (info)
      {
         printf ("\n");
         printf ("Magic number               = %x \n", image->tete->rasMagic);
         printf ("Nbr d'echelle : X colonnes = %u \n", image->tete->rasLargeur);
         printf ("Nbr de ligne  : Y rangees  = %u \n", image->tete->rasHauteur);
         printf ("Nbr de bits par pixel      = %u \n", image->tete->rasProfondeur);
         printf ("Longueur de l'image : X*Y  = %u \n", image->tete->rasLongueur);
         printf ("Type du fichier raster     = %u \n", image->tete->rasType);
         printf ("Type de la 'Color Map'     = %u \n", image->tete->rasMapType);
         printf ("Longueur de la 'Color map' = %u \n", image->tete->rasMapLongueur);
         printf ("\n");
      
      } /* if then */
   
      if (image->tete->rasMagic != RAS_MAGIC)
      {
         printf ("\nInfo : Le fichier '%s' n'est pas de type 'SunRaster'.\n",
                *image->fichiers.fichierS);
         exit (-2);
      
      } /* if then */
   
      if (image->tete->rasType != RT_STANDARD)
      {
         printf ("\nInfo : Le fichier '%s' n'est pas de type 'SunRaster'",
                " standard.\n", *image->fichiers.fichierS);
         exit (-3);
      
      } /* if then */
   
      if (image->tete->rasMapType != RMT_EQUAL_RGB)
      {
         printf ("\nInfo : Le fichier '%s' est de type 'SunRaster' sans tables",
                " de couleurs.\n", *image->fichiers.fichierS);
         exit (-4);
      
      } /* if then */
   
   } /* lireTete */

/* ---------------------------------------------------------------------------
Fonction ECRIRETETE : Cette fonction ecrit la tete de l'image au fichier
                      destination (selon le descripteur fds).

appel : ecrireTete (image)
--------------------------------------------------------------------------- */
   void ecrireTete (Image *image)
   {
   /*===========================================================================
    *          Retour a Big Endian avant la sauvegarde de l'image.
    *==========================================================================*/
      image->tete->rasMagic		 = switchByteOrder(image->tete->rasMagic      );
      image->tete->rasLargeur		 = switchByteOrder(image->tete->rasLargeur    );
      image->tete->rasHauteur		 = switchByteOrder(image->tete->rasHauteur    );
      image->tete->rasProfondeur	 = switchByteOrder(image->tete->rasProfondeur );
      image->tete->rasLongueur	 = switchByteOrder(image->tete->rasLongueur   );
      image->tete->rasType			 = switchByteOrder(image->tete->rasType       );
      image->tete->rasMapType		 = switchByteOrder(image->tete->rasMapType    );
      image->tete->rasMapLongueur = switchByteOrder(image->tete->rasMapLongueur);
   
   
      if (fwrite (image->tete, sizeof(*image->tete), 1, image->fichiers.fds)!= 1)
      {
         printf("\nInfo : Impossible d'ecrire la tete de '%s'.\n",
               *image->fichiers.fichierD);
         exit(-8);
      
      } /* if then */
   
   /*===========================================================================
    *            Retour a Little Endian (donnees en memoire).
    *==========================================================================*/
      image->tete->rasMagic		 = switchByteOrder(image->tete->rasMagic      );
      image->tete->rasLargeur		 = switchByteOrder(image->tete->rasLargeur    );
      image->tete->rasHauteur		 = switchByteOrder(image->tete->rasHauteur    );
      image->tete->rasProfondeur	 = switchByteOrder(image->tete->rasProfondeur );
      image->tete->rasLongueur	 = switchByteOrder(image->tete->rasLongueur   );
      image->tete->rasType			 = switchByteOrder(image->tete->rasType       );
      image->tete->rasMapType		 = switchByteOrder(image->tete->rasMapType    );
      image->tete->rasMapLongueur = switchByteOrder(image->tete->rasMapLongueur);
   
   } /* ecrireTete */


/*===============================================================================
 * Change le format de donnees de Big Endian a Little Endian et vice-versa.
 * Le code ci-dessous est portable, indpendemment de la dimension des donnees
 * utilisees sur la plate-forme ( i.e. 16 bits, 32 bits, 64 bits ).
 *==============================================================================*/
   int switchByteOrder(int data){
   
      unsigned int	data1 = data;
      unsigned int	data2 = data;
      unsigned int	data3 = data;
      unsigned int	data4 = data;
      unsigned int	reversed = 0;
   
      int n_bits = ((sizeof(int) >> 2) * CHAR_BIT); /* limits.h */
      int n_bits_X_3 = n_bits * 3;
   
      reversed |= ((data1 << n_bits_X_3));
      data2   >>= n_bits;
      data2   <<= n_bits_X_3;
      reversed |= data2 >> n_bits;
      data3   <<= n_bits;
      data3   >>= n_bits_X_3;
      reversed |= data3 << n_bits;
      reversed |= data4 >> n_bits_X_3;
   
      return (int)reversed;
   }
